
/* Chrysalide - Outil d'analyse de fichiers binaires
 * access-int.h - prototypes internes pour l'accès à un élément d'expression sous-jacent
 *
 * Copyright (C) 2023 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _ANALYSIS_SCAN_EXPRS_ACCESS_INT_H
#define _ANALYSIS_SCAN_EXPRS_ACCESS_INT_H


#include "access.h"


#include "../expr-int.h"



/* Accès à un élément d'expression sous-jacent (instance) */
struct _GNamedAccess
{
    GScanExpression parent;                 /* A laisser en premier        */

    union
    {
        GRegisteredItem *base;              /* Base de recherche           */
        GRegisteredItem *resolved;          /* Elément ciblé au final      */
        GObject *any;                       /* Accès indistinct            */
    };

    char *target;                           /* Cible dans l'espace         */

    struct _GNamedAccess *next;            /* Evnetuel prochain élément   */

};

/* Accès à un élément d'expression sous-jacent (classe) */
struct _GNamedAccessClass
{
    GScanExpressionClass parent;            /* A laisser en premier        */

};


/* Met en place une expression d'accès. */
bool g_named_access_create(GNamedAccess *, const sized_string_t *);

/* Réduit une expression à une forme plus simple. */
bool _g_named_access_reduce(GNamedAccess *, GScanContext *, GScanScope *, GScanExpression **);



#endif  /* _ANALYSIS_SCAN_EXPRS_ACCESS_INT_H */
